#ifndef msg_h_included_eoieor
#define msg_h_included_eoieor

struct mchan
{
  int fd;
  int bsiz, roff, len, woff;
  unsigned char *buf;

  // for pending messages
  struct msg *mbuf;
  int mbsiz, mblen, mbhead;

  int rW;
  int pending_write;
};

struct msg
{
  int type;
  int ni, ns;
  int iarg[10];
  char *sarg[10];
  int larg[10];
  void (*encoded)(struct msg*);
};

/**
  The mchan struct is allocated by the user. mchan_free() doesn't
  deallocate the mchan struct, just its contents.
 **/
void  mchan_init   (struct mchan *C, int fd, int rW);
void  mchan_free   (struct mchan *C);

/**
  return values:
    0 :  no change in status
    1 :  a message was received into M
   -1 :  there was an error in comms
 **/
int   mchan_recv   (struct mchan *C, struct msg *M);

void  mchan_queue  (struct mchan *C, struct msg *M);
/**
  return values:
    0 :  no change in status
    1 :  queue empty, you may remove fd from the writable watchlist
   -1 :  error in comms 
 **/
int   mchan_send   (struct mchan *C);


/**
 When there is a communication error, the respective channel is no
 longer usable. 

 The user should shutdown and close the socket in this case.
 **/


/** Following functions can be used to set up a socket connection. **/

/**
  The address is numerical. This function returns an fd from which
  a socket descriptor can be read when the connection is ready.
  So, the real socket descriptor is returned by socket_connect().

  Both functions return -1 when there is an error.
 **/
int socket_client(const char *text_address, int port);
int socket_connect(int fd);

/**
  socket_server() creates a server socket and listens to it.
  socket_accept() returns a new data socket connected to a client
 **/
int socket_server(int port, int backlog);
int socket_accept(int fd);

#endif
